<?php
/*
Plugin Name: WordPress File Monitor
Plugin URI: http://mattwalters.net/projects/wordpress-file-monitor/
Description: Monitor files under your WordPress installation for changes
Author: Matt Walters
Version: 1.0.2
Author URI: http://mattwalters.net/
*/ 

if (!class_exists('msw_WordPressMonitor')) {
	class msw_WordPressMonitor {
		var $options = array(
			'scan_interval',
			'from_address',
			'notify_address',
			'site_root',
			'exclude_paths',
			'modification_detection'
		);

		function msw_WordPressMonitor() { $this->__construct(); }
		
		function __construct() {
			add_action("admin_menu", array(&$this,"add_admin_pages"));

			// Assumes your language files will be in the format: wordpress_file_monitor-locationcode.mo
			$wordpress_file_monitor_locale = get_locale();
			$wordpress_file_monitor_mofile = dirname(__FILE__) . "/languages/wordpress_file_monitor-".$wordpress_file_monitor_locale.".mo";
			load_textdomain("wordpress_file_monitor", $wordpress_file_monitor_mofile);
		}

		function add_admin_pages() {
			add_submenu_page('options-general.php', "WordPress File Monitor", "WordPress File Monitor", 10, "WordPress File Monitor", array(&$this,"output_sub_admin_page_0"));
		}

		function update_options($newOptions) {
			foreach ($this->options as $option) {
				$options[$option] = $newOptions[$option];
			}
			if (!get_option('wpfm_options')) { add_option('wpfm_options'); }
			update_option('wpfm_options', maybe_serialize($options));
		}

		function output_sub_admin_page_0() {
			$options = maybe_unserialize(get_option('wpfm_options'));
			?>
			<div class="wrap">
				<h2>WordPress File Monitor Options</h2>
				<br/><br/>
				<form name="msw_wpfm_options" action="<?php echo get_bloginfo('wpurl'); ?>/wp-admin/options-general.php?page=WordPress%20File%20Monitor" method="post">
					<?php if (function_exists('wp_nonce_field')) { wp_nonce_field('wpfm-update-options'); } ?>
					<input type="hidden" name="msw_wpfm_action" value="1" id="msw_wpfm_action">
					<table>
						<tr>
							<td valign="middle"><label for="scan_interval"><?php _e('Scan Interval', 'wordpress_file_monitor'); ?>: </label></td>
							<td valign="middle"><input type="text" name="scan_interval" value="<?php echo @$options['scan_interval']; ?>" id="scan_interval"> (<?php _e('in minutes', 'wordpress_file_monitor'); ?>)</td>
						</tr>
						<tr>
							<td valign="middle"><label for="modification_detection">Detection Method: </label></td>
							<td valign="middle">
								<select name="modification_detection" id="modification_detection">
									<option value="datetime"<?php if (@$options['modification_detection'] == 'datetime') { echo ' selected'; } ?>><?php _e('Modification Date (faster, but less secure)', 'wordpress_file_monitor'); ?></option>
									<option value="md5"<?php if (@$options['modification_detection'] == 'md5') { echo ' selected'; } ?>><?php _e('Hash (more secure, but takes longer)', 'wordpress_file_monitor'); ?></option>
								</select>
							</td>
						</tr>
						<tr>
							<td valign="middle"><label for="from_address"><?php _e('From Address', 'wordpress_file_monitor'); ?>: </label></td>
							<td valign="middle"><input type="text" name="from_address" value="<?php echo @$options['from_address']; ?>" id="from_address"> (<?php _e('for alerts', 'wordpress_file_monitor'); ?>)</td>
						</tr>
						<tr>
							<td valign="middle"><label for="notify_address"><?php _e('Notify Address', 'wordpress_file_monitor'); ?>: </label></td>
							<td valign="middle"><input type="text" name="notify_address" value="<?php echo @$options['notify_address']; ?>" id="notify_address"> (<?php _e('for alerts', 'wordpress_file_monitor'); ?>)</td>
						</tr>
						<tr>
							<td valign="middle"><label for="site_root"><?php _e('Site Root', 'wordpress_file_monitor'); ?>: </label></td>
							<td valign="middle"><input type="text" name="site_root" value="<?php if ($options['site_root'] == '') { echo ABSPATH; } else { echo $options['site_root']; } ?>" id="site_root"> (<?php _e('Default', 'wordpress_file_monitor'); ?>: <?php echo ABSPATH; ?>)</td>
						</tr>
						<tr>
							<td valign="top"><label for="exclude_paths"><?php _e('Exclude Paths', 'wordpress_file_monitor'); ?>: </label></td>
							<td valign="top"><textarea name="exclude_paths" rows="8" cols="40"><?php echo @$options['exclude_paths']; ?></textarea></td>
						</tr>
						<tr>
							<td valign="top">&nbsp;</td>
							<td valign="top">
								<?php _e('Do not include the site root from above. One path per line.', 'wordpress_file_monitor'); ?><br/>
								<br/>
								<?php _e('If you run any kind of cacheing plugins or scripts on your site and the cache files are stored in a folder under the Site Root specified above, ', 'wordpress_file_monitor'); ?><br/>
								<?php _e('it is HIGHLY recommended you exclude the paths to your cache directories.', 'wordpress_file_monitor'); ?>
								<br/><br/>
								<?php _e('Examples', 'wordpress_file_monitor'); ?>:<br/>
								wordpress/wp-content/cache<br/>
								wp-content/uploads
							</td>
						</tr>
					</table>
					<input type="submit" name="Submit" value="<?php _e('Submit', 'wordpress_file_monitor'); ?>" id="Submit">
				</form>
			</div>
			<?php
		}

		function directoryToArray($directory, $recursive=true) {
			$array_items = array();
			if ($handle = opendir($directory)) {
				while (false !== ($file = readdir($handle))) {
					if ($file != "." && $file != "..") {
						if (is_dir($directory. "/" . $file)) {
							if($recursive) {
								$array_items = array_merge($array_items, $this->directoryToArray($directory. "/" . $file, $recursive));
							}
							$file = $directory . "/" . $file;
							$array_items[] = preg_replace("/\/\//si", "/", $file);
						} else {
							$file = $directory . "/" . $file;
							$array_items[] = preg_replace("/\/\//si", "/", $file);
						}
					}
				}
				closedir($handle);
			}
			return $array_items;
		}

		function scan_site() {
			$options = maybe_unserialize(get_option('wpfm_options'));
			$dirListing = $this->directoryToArray($options['site_root']);
			foreach ($dirListing as $item) {
				$excludePaths = explode("\n", $options['exclude_paths']);
				$scanPath = true;
				foreach ($excludePaths as $excludePath) {
					$temp = $options['site_root'] . trim($excludePath);
					if (strpos($item, $temp) !== false) { $scanPath = false; }
				}
				if ($scanPath) {
					if ($options['modification_detection'] == 'md5') {
						$currentDirListing[$item] = md5_file($item);
					} else {
						$currentDirListing[$item] = filemtime($item);
					}
				}
			}
			$previousDirListing = get_option('wpfm_listing');
			if ($previousDirListing) {
				$previousDirListing = maybe_unserialize($previousDirListing);
				// Check for differences
				$diff['addedFiles'] = array_diff_key($currentDirListing, $previousDirListing);
				$diff['removedFiles'] = array_diff_key($previousDirListing, $currentDirListing);
				$diff['changedFiles'] = array_diff($currentDirListing, $previousDirListing);
				foreach ($diff['addedFiles'] as $file=>$v) {
					unset($diff['changedFiles'][$file]);
				}
				foreach ($diff['removedFiles'] as $file=>$v) {
					unset($diff['changedFiles'][$file]);
				}
				update_option('wpfm_listing', maybe_serialize($currentDirListing));
				if (count($diff['addedFiles']) > 0 || count($diff['removedFiles']) > 0 || count($diff['changedFiles']) > 0) {
					$this->notify($diff);
				}
			} else {
				// Save listing, first scan
				add_option('wpfm_listing');
				update_option('wpfm_listing', maybe_serialize($currentDirListing));
			}
		}

		function notify($diff=array()) {
			// Send notifaction email
			$options = maybe_unserialize(get_option('wpfm_options'));
			$toEmail = $options['notify_address'];
			$fromEmail = $options['from_address'];
			$fromName = __('WordPress File Monitor', 'wordpress_file_monitor');
			$headers = "From: " . $fromName . " <" . $fromEmail . ">\r\n";
			$subject = __('WordPress File Monitor: Alert', 'wordpress_file_monitor');
			$body = __('This email is to alert you of the following changes to the file system of your website:', 'wordpress_file_monitor');
			$body .= "\n\n";
			if (count($diff['addedFiles']) > 0) {
				$body .= __('Added:', 'wordpress_file_monitor');
				$body .= "\n";
				foreach ($diff['addedFiles'] as $file=>$timeStamp) {
					$body .= "./" . str_replace($options['site_root'], '', $file) . "\n";
				}
				$body .= "\n\n";
			}
			if (count($diff['removedFiles']) > 0) {
				$body .= __('Removed:', 'wordpress_file_monitor');
				$body .= "\n";
				foreach ($diff['removedFiles'] as $file=>$timeStamp) {
					$body .= "./" . str_replace($options['site_root'], '', $file) . "\n";
				}
				$body .= "\n\n";
			}
			if (count($diff['changedFiles']) > 0) {
				$body .= __('Changed:', 'wordpress_file_monitor');
				$body .= "\n";
				foreach ($diff['changedFiles'] as $file=>$timeStamp) {
					$body .= "./" . str_replace($options['site_root'], '', $file) . "\n";
				}
			}
			mail($toEmail, $subject, $body, $headers);
		}
	}
}

function msw_wpfm_scan() {
	global $msw_wpfm;
	$options = maybe_unserialize(get_option('wpfm_options'));
	$previousScan = get_option('wpfm_previousScan');
	$scanNeeded = false;
	$scanInterval = intval($options['scan_interval']);

	if ($previousScan) {
		if (((time() - $previousScan) / 60) > $scanInterval) {
			$scanNeeded = true;
		}
	} else {
		$scanNeeded = true;
		add_option('wpfm_previousScan');
	}
	if ($scanNeeded) {
		$msw_wpfm->scan_site();
		update_option('wpfm_previousScan', time());
	}
}

function msw_wpfm_admin_processing() {
	if (isset($_POST['msw_wpfm_action'])) {
		global $msw_wpfm;
		check_admin_referer('wpfm-update-options');
		$msw_wpfm->update_options($_POST);
	}
}

function msw_plugin_action_links($links, $file) {
	$plugin_file = 'wordpress-file-monitor/'.basename(__FILE__);
	if ($file == $plugin_file) {
		$settings_link = '<a href="'.get_bloginfo('wpurl').'/wp-admin/options-general.php?page=WordPress%20File%20Monitor">'.__('Settings', 'wordpress_file_monitor').'</a>';
		array_unshift($links, $settings_link);
	}
	return $links;
}

function msw_wpfm_install() {
	$options = array(
		'scan_interval' => 15,
		'from_address' => get_option('admin_email'),
		'notify_address' => get_option('admin_email'),
		'site_root' => ABSPATH,
		'modification_detection' => 'datetime'
	);
	if (!get_option('wpfm_options')) { add_option('wpfm_options'); }
	update_option('wpfm_options', maybe_serialize($options));
}

if (is_file(trailingslashit(ABSPATH.PLUGINDIR).'wordpress_file_monitor.php')) {
	define('MSW_WPFM_FILE', trailingslashit(ABSPATH.PLUGINDIR).'wordpress_file_monitor.php');
} else if (is_file(trailingslashit(ABSPATH.PLUGINDIR).'wordpress-file-monitor/wordpress_file_monitor.php')) {
	define('MSW_WPFM_FILE', trailingslashit(ABSPATH.PLUGINDIR).'wordpress-file-monitor/wordpress_file_monitor.php');
}

if (!$msw_wpfm) { $msw_wpfm = new msw_WordPressMonitor(); }

add_action('init', 'msw_wpfm_admin_processing');
add_action('init', 'msw_wpfm_scan');
add_filter('plugin_action_links', 'msw_plugin_action_links', 10, 2);
register_activation_hook(MSW_WPFM_FILE, 'msw_wpfm_install');
?>